@@ -203,8 +203,8 @@ static direction_t td9[5] = {
 
 static tab_direction_t td[10] =
 {
-  {9, td0},
-  {8, td1},
+  {9,td0},
+  {8,td1},
   {3,td2},
   {5,td3},
   {3,td4},
@@ -246,6 +246,7 @@ static fame_motion_coding_t fourstep_estimation(fame_motion_t *motion,
   int sad_inter, sad_inter4v, mad_inter;
   fame_motion_vector_t subvectors[4];
   int k;
+  fame_motion_coding_t motion_coding;
 
   x = mb_x << 4;
   y = mb_y << 4;
@@ -341,11 +342,14 @@ static fame_motion_coding_t fourstep_estimation(fame_motion_t *motion,
               vectors[2].deviation + vectors[3].deviation;
   count = vectors[0].count + vectors[1].count + vectors[2].count + vectors[3].count;
 
+  motion_coding = motion_inter;
+
   /* inter4v/inter mode decision */
   if((motion->flags & FAME_MOTION_BLOCK_SEARCH) &&
      sad_inter4v + ((count>>1)+1) < sad_inter) {
     /* inter4v prediction */
     sad_inter = sad_inter4v;
+    motion_coding = motion_inter4v;
     for(k = 0; k < 4; k++) {
       vectors[k].dx = subvectors[k].dx;
       vectors[k].dy = subvectors[k].dy;
@@ -355,9 +359,9 @@ static fame_motion_coding_t fourstep_estimation(fame_motion_t *motion,
 
   /* intra/inter mode decision */
   if(mad_inter + count + count < sad_inter)
-    return(motion_intra);
-  else
-    return(motion_inter);
+    motion_coding = motion_intra;
+
+  return(motion_coding);
 }
 
 /*  find_vector                                                              */
