@@ -416,8 +416,10 @@ static int profile_mpeg_encode(fame_profile_t *profile)
   fame_motion_coding_t motion_coding;
   fame_motion_vector_t forward[6];
   unsigned char pattern;
+  int dquant;
   int mv_count = 0;
   int mv_norm = 0;
+  int quant_sum = 0;
 
   /* Test for end of frame */
   if(profile_mpeg->slice_start >= (profile_mpeg->height >> 4))
@@ -451,6 +453,7 @@ static int profile_mpeg_encode(fame_profile_t *profile)
   for (y = profile_mpeg->slice_start; y < slice_end; y++) 
     for (x = 0; x < (profile_mpeg->width >> 4); x++)
     {
+      /* code shape */
       if(profile_mpeg->shape && profile_mpeg->shape->encode_intra_shape)
 	bab_type = profile_mpeg->shape->encode_intra_shape(profile_mpeg->shape,
 							    x, y, &bab, &pattern);
@@ -469,7 +472,16 @@ static int profile_mpeg_encode(fame_profile_t *profile)
       } else
 	motion_coding = motion_intra;
 
+      /* adaptive quantization */
+      if(profile_mpeg->rate && profile_mpeg->rate->local_estimation)
+        dquant = profile_mpeg->rate->local_estimation(profile_mpeg->rate,
+                                                      x, y, blocks,
+                                                      motion_coding);
+      else
+        dquant = 0;
+
       if(motion_coding == motion_intra) {
+
 	profile_mpeg->intra++;
 	/* Code intra macroblock */
 	if(profile_mpeg->encoder &&
@@ -491,41 +503,37 @@ static int profile_mpeg_encode(fame_profile_t *profile)
 	/* Write macroblock */
 	if(profile_mpeg->syntax &&
 	   profile_mpeg->syntax->write_intra_mb)
-	  profile_mpeg->syntax->write_intra_mb(profile_mpeg->syntax,
-					       x, y, blocks,
-					       bab, profile_mpeg->bab_map,
-					       bab_type, 0, pattern);
+          profile_mpeg->quant_scale -= /* dquant may be cancelled */
+            profile_mpeg->syntax->write_intra_mb(profile_mpeg->syntax,
+                                                 x, y, blocks,
+                                                 bab, profile_mpeg->bab_map,
+                                                 bab_type, dquant, pattern);
       } else {
 	profile_mpeg->inter++;
 	/* TODO: check for coded in syntax macroblock depending on error   */
 	/* of motion estimation, for inter blocks only.                    */
 	if(profile_mpeg->search_range_adaptive) {
-	  if(forward[0].dx == forward[1].dx &&
-	     forward[0].dx == forward[2].dx &&
-	     forward[0].dx == forward[3].dx &&
-	     forward[0].dy == forward[1].dy &&
-	     forward[0].dy == forward[2].dy &&
-	     forward[0].dy == forward[3].dy) {
-	    mv_count++;
-	    mv_norm += forward[0].dx*forward[0].dx + forward[0].dy*forward[0].dy;
-	  } else {
+          if(motion_coding == motion_inter4v) {
 	    mv_count += 4;
 	    mv_norm += forward[0].dx*forward[0].dx + forward[0].dy*forward[0].dy;
 	    mv_norm += forward[1].dx*forward[1].dx + forward[1].dy*forward[1].dy;
 	    mv_norm += forward[2].dx*forward[2].dx + forward[2].dy*forward[2].dy;
 	    mv_norm += forward[3].dx*forward[3].dx + forward[3].dy*forward[3].dy;
-	  }
+          } else {
+            mv_count++;
+	    mv_norm += forward[0].dx*forward[0].dx + forward[0].dy*forward[0].dy;
+	  } 
 	}
 
 	/* Code inter macroblock */
 	if(profile_mpeg->encoder &&
 	   profile_mpeg->encoder->encode_inter_mb &&
 	   bab_type != bab_not_coded)
-	  profile_mpeg->encoder->encode_inter_mb(profile_mpeg->encoder,
-						 x, y, blocks,
-						 forward, NULL, motion_coding,
-						 profile_mpeg->quant_scale,
-						 bab_type);
+          profile_mpeg->encoder->encode_inter_mb(profile_mpeg->encoder,
+                                                 x, y, blocks,
+                                                 forward, NULL, motion_coding,
+                                                 profile_mpeg->quant_scale,
+                                                 bab_type);
 
 	if(profile_mpeg->next_coding != 'I')
 	  if(profile_mpeg->decoder &&
@@ -539,20 +547,24 @@ static int profile_mpeg_encode(fame_profile_t *profile)
 	/* Write macroblock */
 	if(profile_mpeg->syntax &&
 	   profile_mpeg->syntax->write_inter_mb)
-	  profile_mpeg->syntax->write_inter_mb(profile_mpeg->syntax,
-					       x, y, blocks,
-					       bab, profile_mpeg->bab_map,
-					       bab_type, 0, pattern,
-					       forward, NULL, motion_coding);
+          profile_mpeg->quant_scale -= /* dquant may be cancelled */
+            profile_mpeg->syntax->write_inter_mb(profile_mpeg->syntax,
+                                                 x, y, blocks,
+                                                 bab, profile_mpeg->bab_map,
+                                                 bab_type, dquant, pattern,
+                                                 forward, NULL, motion_coding);
       }
+
+      quant_sum += profile_mpeg->quant_scale;
     }
+  
+#ifdef HAS_MMX
+  asm("emms");
+#endif
 
   if(mv_count) {
     /* adapt search range according to MV standard deviation */
     mv_norm /= mv_count;
-#ifdef HAS_MMX
-    asm("emms");
-#endif
     mv_norm = (int) sqrt(mv_norm);
 
     if(profile_mpeg->search_range < 3*mv_norm && 
@@ -567,6 +579,9 @@ static int profile_mpeg_encode(fame_profile_t *profile)
   /* End of slice */
   if(profile_mpeg->syntax && profile_mpeg->syntax->end_slice)
     profile_mpeg->syntax->end_slice(profile_mpeg->syntax);
+
+  profile_mpeg->quant_avg = (float) quant_sum / 
+    ((slice_end - profile_mpeg->slice_start) * (profile_mpeg->width >> 4));
   
   /* Return the number of bytes encoded */
   if(profile_mpeg->syntax && profile_mpeg->syntax->flush)
@@ -671,10 +686,10 @@ static void profile_mpeg_leave(fame_profile_t *profile,
 		100*profile_mpeg->inter/
 		(profile_mpeg->intra+profile_mpeg->inter));
     }
-    FAME_INFO("%dkbits/s quality %d%% range %d %c frame #%d\033[K\r",
+    FAME_INFO("%dkbits/s quality %.02f%% range %d %c frame #%d\033[K\r",
 	      (profile_mpeg->total * profile_mpeg->fps_num * 8) / 
 	      (profile_mpeg->fps_den * 1000),
-	      (31 - profile_mpeg->quant_scale) * 100 / 30,
+	      (31 - profile_mpeg->quant_avg) * 100 / 30,
 	      profile_mpeg->search_range,
 	      profile_mpeg->current_coding,
 	      profile_mpeg->frame_number);
@@ -683,7 +698,7 @@ static void profile_mpeg_leave(fame_profile_t *profile,
   if(profile_mpeg->monitor && profile_mpeg->monitor->leave)
     profile_mpeg->monitor->leave(profile_mpeg->monitor, 
 				 profile_mpeg->total * 8,
-				 profile_mpeg->quant_scale);
+				 profile_mpeg->quant_avg);
 
   if(stats)
     *stats = *profile_mpeg->frame_stats;
