@@ -88,27 +88,27 @@ static void mpeg1_predict_vector(fame_syntax_t *syntax,
 static void mpeg1_compute_chrominance_vectors(fame_syntax_t *syntax,
 					      fame_motion_vector_t *vectors,
 					      unsigned char pattern);
-static void mpeg1_write_intra_mb(fame_syntax_t *syntax,
-				 int mb_x,
-				 int mb_y,
-				 short *blocks[6],
-				 unsigned char *bab,
-				 unsigned char *bab_map,
-				 fame_bab_t bab_type,
-				 unsigned char dquant,
-				 unsigned char pattern);
-static void mpeg1_write_inter_mb(fame_syntax_t *syntax,
-				 int mb_x,
-				 int mb_y,
-				 short *blocks[6],
-				 unsigned char *bab,
-				 unsigned char *bab_map,
-				 fame_bab_t bab_type,
-				 unsigned char dquant,
-				 unsigned char pattern,
-				 fame_motion_vector_t *forward,
-				 fame_motion_vector_t *backward,
-				 fame_motion_coding_t motion_coding);
+static int mpeg1_write_intra_mb(fame_syntax_t *syntax,
+                                int mb_x,
+                                int mb_y,
+                                short *blocks[6],
+                                unsigned char *bab,
+                                unsigned char *bab_map,
+                                fame_bab_t bab_type,
+                                int dquant,
+                                unsigned char pattern);
+static int mpeg1_write_inter_mb(fame_syntax_t *syntax,
+                                int mb_x,
+                                int mb_y,
+                                short *blocks[6],
+                                unsigned char *bab,
+                                unsigned char *bab_map,
+                                fame_bab_t bab_type,
+                                int dquant,
+                                unsigned char pattern,
+                                fame_motion_vector_t *forward,
+                                fame_motion_vector_t *backward,
+                                fame_motion_coding_t motion_coding);
 
 FAME_CONSTRUCTOR(fame_syntax_mpeg1_t)
 {
@@ -417,6 +417,7 @@ static void mpeg1_start_slice(fame_syntax_t *syntax,
     vpos * syntax_mpeg1->mb_width - 1;
   syntax_mpeg1->slice_length = length;
   syntax_mpeg1->previous_coding = motion_intra;
+  syntax_mpeg1->qscale = qscale;
   /* new slice starting at vpos      */
   bitbuffer_write(&syntax_mpeg1->buffer, SLICE_BASE_CODE + vpos, 32);
   /* quantization scale              */
@@ -491,7 +492,7 @@ static void mpeg1_block_intra(fame_syntax_t *syntax, short *block, fame_vlc_t co
 	 address multiple of 4 bytes (at least), we are sure that sp != 0xffff and thus sp+1 will
 	 never be zero. We then retrieve sp at the end for it is needed by 'pop' instructions.
      */
-
+    /* TODO : echange the role of edx and esp */
     __asm__ __volatile__ ("pushl %%ebx\n"             /* save ebx            */
 			  "pushl %%ebp\n"             /* save stack pointer  */
 			  "inc %%sp\n"                /* make sure sp != 0   */
@@ -581,15 +582,15 @@ static void mpeg1_block_intra(fame_syntax_t *syntax, short *block, fame_vlc_t co
   buffer->shift = shift;
 }
 
-static void mpeg1_write_intra_mb(fame_syntax_t *syntax,
-				 int mb_x,
-				 int mb_y,
-				 short *blocks[6],
-				 unsigned char *bab,
-				 unsigned char *bab_map,
-				 fame_bab_t bab_type,
-				 unsigned char dquant,
-				 unsigned char pattern)
+static int mpeg1_write_intra_mb(fame_syntax_t *syntax,
+                                int mb_x,
+                                int mb_y,
+                                short *blocks[6],
+                                unsigned char *bab,
+                                unsigned char *bab_map,
+                                fame_bab_t bab_type,
+                                int dquant,
+                                unsigned char pattern)
 {
   fame_syntax_mpeg1_t *syntax_mpeg1 = FAME_SYNTAX_MPEG1(syntax);
   int incr;
@@ -624,6 +625,12 @@ static void mpeg1_write_intra_mb(fame_syntax_t *syntax,
     break;
   }
 
+  /* dquant */
+  if(dquant) {
+    syntax_mpeg1->qscale += dquant;
+    bitbuffer_write(&syntax_mpeg1->buffer, syntax_mpeg1->qscale, 5);
+  }
+
   mpeg1_block_intra(syntax, blocks[0], encode_ydc_table, &syntax_mpeg1->y_dc_pred);
   mpeg1_block_intra(syntax, blocks[1], encode_ydc_table, &syntax_mpeg1->y_dc_pred);
   mpeg1_block_intra(syntax, blocks[2], encode_ydc_table, &syntax_mpeg1->y_dc_pred);
@@ -633,6 +640,8 @@ static void mpeg1_write_intra_mb(fame_syntax_t *syntax,
 
   /* reset the motion predictors */
   syntax_mpeg1->mv_pred.dx = syntax_mpeg1->mv_pred.dy = 0;
+
+  return(0);
 }
 
 static void mpeg1_block_inter(fame_syntax_t *syntax, short *block)
@@ -728,18 +737,18 @@ static void inline mpeg1_write_vector(fame_syntax_t *syntax, short delta)
   }
 }
 
-static void mpeg1_write_inter_mb(fame_syntax_t *syntax,
-				 int mb_x,
-				 int mb_y,
-				 short *blocks[6],
-				 unsigned char *bab,
-				 unsigned char *bab_map,
-				 fame_bab_t bab_type,
-				 unsigned char dquant,
-				 unsigned char pattern,
-				 fame_motion_vector_t *forward,
-				 fame_motion_vector_t *backward,
-				 fame_motion_coding_t motion_coding)
+static int mpeg1_write_inter_mb(fame_syntax_t *syntax,
+                                int mb_x,
+                                int mb_y,
+                                short *blocks[6],
+                                unsigned char *bab,
+                                unsigned char *bab_map,
+                                fame_bab_t bab_type,
+                                int dquant,
+                                unsigned char pattern,
+                                fame_motion_vector_t *forward,
+                                fame_motion_vector_t *backward,
+                                fame_motion_coding_t motion_coding)
 {
   fame_syntax_mpeg1_t *syntax_mpeg1 = FAME_SYNTAX_MPEG1(syntax);
   int incr;
@@ -747,6 +756,7 @@ static void mpeg1_write_inter_mb(fame_syntax_t *syntax,
   int coded[6];
   int cbp;
   int i, j;
+  int retval = 1;
 
   incr = mb_y * syntax_mpeg1->mb_width + mb_x - syntax_mpeg1->prev_mb_addr;
 
@@ -794,7 +804,7 @@ static void mpeg1_write_inter_mb(fame_syntax_t *syntax,
     /* reset the motion predictors */
     syntax_mpeg1->mv_pred.dx = syntax_mpeg1->mv_pred.dy = 0;
     /* skip macroblock */
-    return; 
+    return(dquant); 
   }
   else
     motion_forward = 1; /* force coding of block */
@@ -832,6 +842,14 @@ static void mpeg1_write_inter_mb(fame_syntax_t *syntax,
     break;
   }
 
+  /* dquant */
+  if(cbp && dquant) {
+    syntax_mpeg1->qscale += dquant;
+    bitbuffer_write(&syntax_mpeg1->buffer, syntax_mpeg1->qscale, 5);
+    retval = 0;
+  } else
+    retval = dquant;
+
   /* motion vectors */
   if(motion_forward) {
     mpeg1_write_vector(syntax, forward[0].dx - syntax_mpeg1->mv_pred.dx);
@@ -863,6 +881,8 @@ static void mpeg1_write_inter_mb(fame_syntax_t *syntax,
   /* reset the predictors to their original values */
   syntax_mpeg1->y_dc_pred = 128;
   syntax_mpeg1->cr_dc_pred = syntax_mpeg1->cb_dc_pred = 128;
+
+  return(retval);
 }
 
 static void mpeg1_compute_chrominance_vectors(fame_syntax_t *syntax,
